#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
import math
from pyromaths.ex import LegacyExercise


def nodesep(ligne):
    """
    Défini les valeurs nodesep : 0 pour une extrémité, -0.5 pour une continuité
    @param ligne: droite, demi-droite, segment
    @type ligne: string
    """

    if ligne == 'une droite':
        retour = ['-6', '-6']
    elif ligne == 'une demi-droite':
        retour = ['0', '-6']
    else:
        retour = ['0', '0']
    return retour


def choix_points(n):
    """
    choisit n points parmi A, B, C, ..., Z
    @param n: nombre de points à choisir
    @type n: integer
    """

    points = [chr(i + 65) for i in range(26)]
    liste = []
    for i in range(n):
        liste.append(points.pop(random.randrange(len(points))))
    return liste


def choix_ligne(n):
    """
    Retourne n propositions parmi droite, segment et demi-droite
    @param n: nombre de propositions
    @type n: interger
    """

    lignes = ['une droite', 'une demi-droite', 'un segment']
    (liste_lignes, retour) = ([], [])
    for dummy in range((n - 1) // len(lignes) + 1):
        liste_lignes.extend(lignes)
    for dummy in range(n):
        retour.append(liste_lignes.pop(random.randrange(len(liste_lignes))))
    return retour


def symboles(ligne):
    """
    Retourne les couples (), [] ou [) correspondant au type de ligne
    @param ligne: droite, demi-droite ou segment
    @type ligne: string
    """

    if ligne == 'une droite':
        retour = ['(', ')']
    elif ligne == 'une demi-droite':
        retour = ['[', ')']
    else:
        retour = ['[', ']']
    return retour


def prepare_tuple(lpoints, ligne):
    """
    Prepare deux tuples pour permettre l'affichage de la question et
    de la solution
    @param lpoints: les points de la figure
    @type lpoints: liste de lettres
    @param ligne: droite, demi-droite ou segment
    @type ligne: string
    """

    (retour_exo, retour_sol) = ([], [])

    # choix des deux points permettant de tracer la ligne :

    templist = [i for i in range(len(lpoints))]
    deuxpoints = []
    for i in range(2):
        deuxpoints.append(lpoints[templist.pop(random.randrange(len(templist)))])

    # choix des symbole correspondant à la ligne :

    lsymboles = symboles(ligne)
    retour_sol.append(lsymboles[0])
    retour_sol.extend(deuxpoints)
    retour_sol.append(lsymboles[1])
    retour_sol.append(ligne)

    # choix des trous pour l'exercice :

    alea = random.randrange(3)
    if alea > 1:
        retour_exo = ['\\ldots', '\\ldots', '\\ldots', '\\ldots',
                      '\\dotfill']
    elif alea > 0:
        retour_exo = ['\\ldots']
        retour_exo.extend(retour_sol[1:3])
        retour_exo.extend(['\\ldots', retour_sol[4]])
    else:
        retour_exo = retour_sol[:4]
        retour_exo.append('\\dotfill')
    return (tuple(retour_exo), tuple(retour_sol))


def tex_figure(liste, lpoints, points_coord, nodesep=0):
    """
    Écrit dans un fichier tex la construction de 3 points et éventuellement
    une droite, une demi-droite ou un segment.
    @param liste: liste d'exos ou corrigés
    @type liste: liste
    @param lpoints: liste de 3 points
    @type lpoints: liste de 3 strings
    @param nodesep: liste des dépassements pour pstricks
    @type nodesep: liste de 2 strings
    """
    liste.append('\\begin{pspicture*}(-0.5,0.2)(4.5,2.2)')
    liste.append('\\psset{PointSymbol=x}')
    liste.append('\\pstGeonode[PosAngle=90](0.5,%s){%s}(2,%s){%s}(3.5,%s){%s}' % 
               points_coord)
    if nodesep:
        liste.append('\\pstLineAB[nodesepA=%s, nodesepB=%s]{%s}{%s}' % 
                   tuple(nodesep))
    liste.append('\\end{pspicture*}\\tabularnewline')

def coord_points(lpoints):
    """Définit les ordonnées de trois points nommés dont les noms sont dans lpoints"""
    ordonnees = [random.randrange(5, 16) / 10 for i in range(3)]
    while abs(2 * ordonnees[1] - ordonnees[0] - ordonnees[2]) < .5:
        ordonnees = [random.randrange(5, 16) / 10 for i in range(3)]
    random.shuffle(ordonnees)
    for i in range(3):
        ordonnees.insert(2 * i + 1, lpoints[i])
    return tuple(ordonnees)

def tex_ligne_tableau(exo, cor, ligne):
    """
    Écrit une ligne de tableau dans un fichier tex
    @param exo: fichier d'exercices
    @type exo: file
    @param cor: fichier de corrections
    @type cor: file
    @param ligne: droite, demi-droite ou segment
    @type ligne: string
    """

    lpoints = choix_points(3)
    (exer, solution) = prepare_tuple(lpoints, ligne)
    exo.append('$%s %s%s %s$ est %s &' % 
             exer)
    cor.append('$%s %s%s %s$ est %s &' % 
             solution)
    lnodesep = nodesep(ligne)
    lnodesep.extend(solution[1:3])
    points_coord = coord_points(lpoints)
    if exer != ('\\ldots', '\\ldots', '\\ldots', '\\ldots', '\\dotfill'):
        tex_figure(exo, lpoints, points_coord)
    else:
        tex_figure(exo, lpoints, points_coord, lnodesep)
    tex_figure(cor, lpoints, points_coord, lnodesep)
    exo.append('\\hline')
    cor.append('\\hline')


def _Droites():
    """
    Écrit les 5 lignes du tableau
    @param exo: fichier d'exercices
    @type exo: file
    @param cor: fichier de corrections
    @type cor: file
    """
    exo = ["\\exercice", u"Compléter les pointillés et les figures :\\par",
            '\\renewcommand{\\tabularxcolumn}[1]{m{#1}}',
            '\\begin{tabularx}{\\linewidth}{|X|>{\\centering}m{5cm}|}',
            '\\hline',
            u'\\textbf{phrase} & \\textbf{Figure} \\tabularnewline \\hline']
    cor = ["\\exercice*", u"Compléter les pointillés et les figures :\\par",
            '\\renewcommand{\\tabularxcolumn}[1]{m{#1}}',
            '\\begin{tabularx}{\\linewidth}{|X|>{\\centering}m{5cm}|}',
            '\\hline',
            u'\\textbf{Phrase} & \\textbf{Figure} \\tabularnewline \\hline']

    line = choix_ligne(5)
    for i in range(5):
        tex_ligne_tableau(exo, cor, line[i])

    exo.append('\\end{tabularx}')
    cor.append('\\end{tabularx}')
    return (exo, cor)

class Droites(LegacyExercise):
    """Droites, demi-droites, segments"""

    tags = ["Sixième"]
    function = _Droites


#------------------------------------------------------------------------------
# Parallèles et perpendiculaires
#------------------------------------------------------------------------------
# perp à (ac) passant par b => val2=('a', 'c', 'a', 'c', 'b', 'b', 'b', 'a'
# para à (ab) passant apr d => val2=('a', 'b', 'a', 'b', 'd', 'd')


def fig_perp(points, coor, solution=0, per=[], par=[]):
    val_enonce = (
        points[0], points[1], coor[0], coor[1], coor[2], coor[3],
        points[2], points[3], coor[4], coor[5], coor[6], coor[7],)
    pts = ('a', 'b', 'c', 'd')
    text = \
        """  \\begin{pspicture*}(-4,-4)(4,4)
    \psset{PointSymbol=x}
    \pstGeonode[PointName={%s,%s}](%s;%s){a}(%s;%s){b}
    \pstGeonode[PointName={%s,%s}](%s; %s){c}(%s; %s){d}""" % \
        val_enonce
    if solution:
        val_soluce = (
            pts[per[0]], pts[per[1]], pts[per[0]], pts[per[1]],
            pts[per[2]], pts[per[2]], pts[per[2]], pts[per[0]],
            pts[par[0]], pts[par[1]], pts[par[0]], pts[par[1]],
            pts[par[2]], pts[par[2]])

        text = text + \
            """    \pstLineAB[nodesep=-4, linecolor=DarkBlue]{%s}{%s}
    \pstProjection[PointName=none]{%s}{%s}{%s}[e]\pstLineAB[nodesep=-7, linecolor=DarkBlue]{%s}{e}
    \pstRightAngle[, linecolor=DarkBlue]{%s}{e}{%s}
    \pstLineAB[nodesep=-4, linecolor=DarkRed]{%s}{%s}
    \pstTranslation[PointName=none,PointSymbol=none]{%s}{%s}{%s}[f]
    \pstLineAB[nodesep=-7, linecolor=DarkRed]{%s}{f}
  \end{pspicture*}""" % \
            val_soluce
    return text


def noms_sommets(nb):  # renvoie nb noms de sommets
    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(chr(listenb.pop(random.randrange(26 - i))))
    listepts.sort()
    return tuple(listepts)


def cree_coordonnees(longueur=3):
    from math import floor
    alpha = random.randrange(180)
    k0 = random.randrange(50, 100) / 100
    a0 = alpha + random.randrange(30, 120)
    k1 = random.randrange(50, 100) / 100
    a1 = alpha + random.randrange(210, 300)
    return (longueur,alpha, longueur, alpha + 180, floor((k0* 10) * longueur) / 10, a0,
            floor((k1 * 10) * longueur) / 10, a1)


def enonce_perp(exo, cor):
    coor = cree_coordonnees(3)
    noms = noms_sommets(4)
    par, per = [], []
    lval = [0, 1, 2, 3]
    for dummy in range(3):
        par.append(lval.pop(random.randrange(len(lval))))
    while per == [] or (par[0], par[1]) == (per[0], per[1]) or \
            (par[0], par[1]) == (per[1], per[0]) :
        lval = [0, 1, 2, 3]
        per = []
        for dummy in range(3):
            per.append(lval.pop(random.randrange(len(lval))))
    exo.append(fig_perp(noms, coor))
    cor.append(fig_perp(noms, coor, 1, per, par))
    exo.append('\end{pspicture*}\\par\n\\begin{enumerate}')
    cor.append('\\par\n\\begin{enumerate}')
    s_per = u"\\item Tracer la droite perpendiculaire à la droite $(%s%s)$ passant par $%s$"
    s_par = u"\\item Tracer la droite parallèle à la droite $(%s%s)$ passant par $%s$"
    s_per = s_per % (noms[per[0]], noms[per[1]], noms[per[2]])
    s_par = s_par % (noms[par[0]], noms[par[1]], noms[par[2]])
    if random.randrange(2):
        exo.append(s_par)
        cor.append(s_par)
        exo.append(s_per)
        cor.append(s_per)
    else:
        exo.append(s_per)
        cor.append(s_per)
        exo.append(s_par)
        cor.append(s_par)
    exo.append('\\end{enumerate}')
    cor.append('\\end{enumerate}')


def _Perpendiculaires():
    exo = ["\\exercice", u"Réaliser les figures suivantes :\\par", '\\begin{multicols}{2}']
    cor = ["\\exercice*", u"Réaliser les figures suivantes :\\par", '\\begin{multicols}{2}']

    enonce_perp(exo, cor)

    exo.append('\\columnbreak')
    cor.append('\\columnbreak')

    enonce_perp(exo, cor)

    exo.append('\\end{multicols}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class Perpendiculaires(LegacyExercise):
    """Droites perpendiculaires et parallèles"""

    tags = ["Sixième"]
    function = _Perpendiculaires


#------------------------------------------------------------------------------
# Propriétés
#------------------------------------------------------------------------------


def fonction(angle, xa, ya, dist=0, droite='par'):
    """
    Retourne une fonction à utiliser avec psplot
    @param angle: compris entre 1 et 89° ou 91 et 179°. Angle entre la droite et l'axe des abscisses
    @type angle:
    @param xa: abscisse d'un point de la droite
    @type xa:
    @param ya: ordonnée d'un point de la droite
    @type ya:
    @param dist: distance entre l'origine et la droite
    @type dist:
    @param droite: 'par' pour une parallèle et 'per' pour une perpendiculaire
    """

    angle_rad = (angle * math.pi) / 180
    if droite == 'par':
        coef = math.floor(math.tan(angle_rad) * 1000) / 1000
        ord_or = math.floor((ya - xa * math.tan(angle_rad) - dist / math.cos(angle_rad)) * 1000) / 1000
        return '{x %s mul %s add}' % (coef, ord_or)
    else:
        coef = math.floor(-1000 / math.tan(angle_rad)) / 1000
        return '{x %s mul}' % coef


def PointInter(angle, xa, ya, dist=0):
    angle_rad = (angle * math.pi) / 180
    coef1 = math.floor(math.tan(angle_rad) * 1000) / 1000
    ord_or1 = math.floor((ya - xa * math.tan(angle_rad)) - dist / math.cos(angle_rad) * 1000) / 1000
    coef2 = math.floor(-1000 / math.tan(angle_rad)) / 1000
    x = ord_or1 // (coef2 - coef1)
    y = x * coef2
    return ',PosAngle=%s](%s,%s)' % (45 + angle, math.floor(x * 1000) / 1000, math.floor(y * 1000) / 1000)


def Points(angle, xa, ya, dist=0):
    angle_rad = (angle * math.pi) / 180
    coef = math.floor(math.tan(angle_rad) * 1000) / 1000
    ord_or = math.floor((ya - xa * math.tan(angle_rad) - dist / math.cos(angle_rad)) * 1000) / 1000
    lpos = []
    if -1.5 < -2 * coef + ord_or < 1.5:
        x = -1.5
        y = math.floor((x * coef + ord_or) * 1000) / 1000
        lpos.append('(%s,%s)' % (x, y))
    if -1.5 < 2 * coef + ord_or < 1.5:
        x = 1.5
        y = math.floor((x * coef + ord_or) * 1000) / 1000
        lpos.append('(%s,%s)' % (x, y))
    if -2.1 < (1.5 - ya + dist / math.cos(angle_rad) + xa * math.tan(angle_rad)) / math.tan(angle_rad) < 2.1:
        y = 1.1
        x = math.floor(((y - ya + dist / math.cos(angle_rad) + xa * math.tan(angle_rad)) / math.tan(angle_rad)) * 1000) / 1000
        lpos.append('(%s,%s)' % (x, y))
    if -2.1 < (-1.5 - ya + dist / math.cos(angle_rad) + xa * math.tan(angle_rad)) / math.tan(angle_rad) < 2.1:
        y = -1.1
        x = math.floor(((y - ya + dist / math.cos(angle_rad) + xa * math.tan(angle_rad)) / math.tan(angle_rad)) * 1000) / 1000
        lpos.append('(%s,%s)' % (x, y))
    return lpos


def figure(angle, xa, ya, dist, lpoints, noms, par_per, dist2=0):
    """

    @param angle:
    @param xa:
    @param ya:
    @param dist:
    @param lpoints:
    @param noms: 1: nomme la droite (AB)
                 2: nomme la droite (d1)
    @param par_per: 1: parallèles + perpendiculaires
                    2: 3 parallèles
                    3: 2 perpendiculaires
    """

    ltxt = []
    ltxt.append('\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)')
    ltxt.append('\\footnotesize')
    if par_per < 3:
        ltxt.append('\\psplot[linewidth=1.5\\pslinewidth]{-2.1}{2.1}%s' % 
                    fonction(angle, xa, ya))
        ltxt.append('\\psplot[linewidth=1.5\\pslinewidth]{-2.1}{2.1}%s' % 
                    fonction(angle, xa, ya, dist))
    else:
        ltxt.append('\\psplot{-2.1}{2.1}%s' % fonction(angle, xa, ya))
        ltxt.append('\\psplot{-2.1}{2.1}%s' % fonction(angle, xa, ya,
                    dist))
    if par_per == 2:
        ltxt.append('\\psplot[linewidth=1.5\\pslinewidth]{-2.1}{2.1}%s' % 
                    fonction(angle, xa, ya, dist2))
    else:
        ltxt.append('\\psplot{-2.1}{2.1}%s' % fonction(angle, xa, ya,
                    droite='per'))
    if noms:  # nomme les droites par deux points
        if par_per != 2:  # 2 points d'intersection
            ltxt.append('\\pstGeonode[PointSymbol={none,x},PointName={%s,%s} %s{i1}%s{a1}' % 
                        (lpoints[0], lpoints[1], PointInter(angle, xa,
                        ya), Points(angle, xa, ya)[0]))
            ltxt.append('\\pstGeonode[PointSymbol={none,x},PointName={%s,%s} %s{i2}%s{b1}' % 
                        (lpoints[2], lpoints[3], PointInter(angle, xa,
                        ya, dist), Points(angle, xa, ya, dist)[0]))
            ltxt.append('\\pstGeonode[PointSymbol=none,PointName=none]%s{c1}%s{c2}' % 
                        (Points(angle + 90, 0, 0)[0], Points(angle + 90,
                        0, 0)[1]))
        else:

                # pas de point d'intersection

            pts = Points(angle, xa, ya)
            ltxt.append('\\pstGeonode[PointSymbol=x,PosAngle=%s,PointName={%s,%s}]%s{a1}%s{a2}' % 
                        (angle + 45, lpoints[0], lpoints[1], pts[0], pts[1]))
            pts = Points(angle, xa, ya, dist)
            ltxt.append('\\pstGeonode[PointSymbol=x,PosAngle=%s,PointName={%s,%s}]%s{b1}%s{b2}' % 
                        (angle - (45.0 * dist) / abs(dist), lpoints[2],
                        lpoints[3], pts[0], pts[1]))
            pts = Points(angle, xa, ya, dist2)
            ltxt.append('\\pstGeonode[PointSymbol=x,PosAngle=%s,PointName={%s,%s}]%s{c1}%s{c2}' % 
                        (angle - (45.0 * dist2) / abs(dist2), lpoints[4],
                        lpoints[5], pts[0], pts[1]))
    else:

        # nomme les droites (d_1), ...

        if par_per != 2:  # 2 points d'intersection
            ltxt.append('\\pstGeonode[PointSymbol=none,PointName={none,%s} %s{i1}%s{a1}' % 
                        (lpoints[0], PointInter(angle, xa, ya), Points(angle,
                        xa, ya)[0]))
            ltxt.append('\\pstGeonode[PointSymbol=none,PointName={none,%s} %s{i2}%s{b1}' % 
                        (lpoints[1], PointInter(angle, xa, ya, dist),
                        Points(angle, xa, ya, dist)[0]))
            ltxt.append('\\pstGeonode[PointSymbol=none,PointName={none,%s}]%s{c1}%s{c2}' % 
                        (lpoints[2], Points(angle + 90, 0, 0)[0], Points(angle + 
                        90, 0, 0)[1]))
        else:
            pts = Points(angle, xa, ya)
            ltxt.append('\\pstGeonode[PointSymbol=none,PosAngle=%s,PointName={%s,none}]%s{a1}%s{a2}' % 
                        (angle + 45, lpoints[0], pts[0], pts[1]))
            pts = Points(angle, xa, ya, dist)
            ltxt.append('\\pstGeonode[PointSymbol=none,PosAngle=%s,PointName={%s,none}]%s{b1}%s{b2}' % 
                        (angle - (45.0 * dist) / abs(dist), lpoints[1],
                        pts[0], pts[1]))
                        # FIXME list index out of range
            pts = Points(angle, xa, ya, dist2)
            ltxt.append('\\pstGeonode[PointSymbol=none,PosAngle=%s,PointName={%s,none}]%s{c1}%s{c2}' % 
                        (angle - (45.0 * dist2) / abs(dist2), lpoints[2],
                        pts[0], pts[1]))
    if par_per != 2:
        if angle < 90:
            ltxt.append('\\pstRightAngle[RightAngleSize=.2]{c1}{i1}{a1}')
        else:
            ltxt.append('\\pstRightAngle[RightAngleSize=.2]{c2}{i1}{a1}')
    if par_per == 3:
        if angle < 90:
            ltxt.append('\\pstRightAngle[RightAngleSize=.2]{c1}{i2}{b1}')
        else:
            ltxt.append('\\pstRightAngle[RightAngleSize=.2]{c2}{i2}{b1}')
    ltxt.append('\\end{pspicture*}')
    return ltxt


def valeurs_figures(par_per):
    noms = random.randrange(2)
    if noms:
        lpoints = noms_sommets(6)
    else:
        lindices = [1, 2, 3]
        lpoints = []
        for dummy in range(3):
            lpoints.append('(d_%s)' % lindices.pop(random.randrange(len(lindices))))
    angle = random.randrange(1, 90) + 90 * random.randrange(2)
    xa = random.randrange(-5, 5) / 10
    ya = random.randrange(-3, 3) / 10
    if random.randrange(2):
        dist = random.randrange(4, 9) / 10
    else:
        dist = -random.randrange(4, 9) / 10
    if par_per == 2:
        if dist > 0:
            dist2 = -random.randrange(4, 9) / 10
        else:
            dist2 = random.randrange(4, 9) / 10
        return (angle, xa, ya, dist, lpoints, noms, dist2)
    else:
        return (angle, xa, ya, dist, lpoints, noms)


def enonce_prop(exo, cor):
    exo.append('\\renewcommand{\\tabularxcolumn}[1]{m{#1}}')
    exo.append('\\begin{tabularx}{\\textwidth}[t]{|m{3cm}|m{4cm}|X|m{3cm}|}')
    exo.append('\\hline')
    exo.append(u'\\multicolumn{1}{|c|}{\\bf Données} & \\multicolumn{1}{|c|}{\\bf Figure codée}')
    exo.append(u'& \\multicolumn{1}{|c|}{\\bf Propriété} & \\multicolumn{1}{|c|}{\\bf Conclusion}\\\\')
    cor.append('\\renewcommand{\\tabularxcolumn}[1]{m{#1}}')
    cor.append('\\begin{tabularx}{\\textwidth}[t]{|m{3cm}|m{4cm}|X|m{3cm}|}')
    cor.append('\\hline')
    cor.append(u'\\multicolumn{1}{|c|}{\\bf Données} & \\multicolumn{1}{|c|}{\\bf Figure codée}')
    cor.append(u'& \\multicolumn{1}{|c|}{\\bf Propriété} & \\multicolumn{1}{|c|}{\\bf Conclusion}\\\\')
    ltypes = [1, 2, 3]
    lexos = []
    for i in range(3):
        lexos.append(ltypes.pop(random.randrange(len(ltypes))))
    for i in range(3):
        exo.append('\\hline')
        cor.append('\\hline')
        v = valeurs_figures(lexos[i])
        if lexos[i] == 2:
            if v[5]:  # noms de la forme (AB), on ajoute des parenthèses
                exo.append('''$(%s%s)//(%s%s)$\\par et\\par $(%s%s)//(%s%s)$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                         (v[4][0], v[4][1], v[4][2], v[4][3], v[4][0], v[4][1],
                         v[4][4], v[4][5]))
                cor.append('$(%s%s)//(%s%s)$\\par et\\par $(%s%s)//(%s%s)$ & ' % 
                         (v[4][0], v[4][1], v[4][2], v[4][3], v[4][0], v[4][1],
                         v[4][4], v[4][5]))
            else:
                exo.append('''$%s//%s$\\par et\\par $%s//%s$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                         (v[4][0], v[4][1], v[4][0], v[4][2]))
                cor.append('$%s//%s$\\par et\\par $%s//%s$ & ' % (v[4][0],
                         v[4][1], v[4][0], v[4][2]))
            cor.append('%s & ' % ('\n').join(figure(v[0], v[1], v[2],
                     v[3], v[4], v[5], lexos[i], v[6])))  # eror out of range in figure
            cor.append(u'Si deux droites sont parallèles, alors toute parallèle à l\'une est parallèle à l\'autre. &')
            if v[5]:
                cor.append('$(%s%s)//(%s%s)$ \\\\\n  \\hline' % (v[4][2],
                         v[4][3], v[4][4], v[4][5]))
            else:
                cor.append('$%s//%s$ \\\\\n  \\hline' % (v[4][1], v[4][2]))
        else:

            fig = random.randrange(2)
            if lexos[i] == 1:
                if v[5]:
                    if not fig:
                        exo.append('''$(%s%s)//(%s%s)$\\par et\\par $(%s%s)\\perp(%s%s)$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                                 (v[4][0], v[4][1], v[4][2], v[4][3], v[4][0],
                                 v[4][1], v[4][0], v[4][2]))
                    cor.append('$(%s%s)//(%s%s)$\\par et\\par $(%s%s)\\perp(%s%s)$ &' % 
                             (v[4][0], v[4][1], v[4][2], v[4][3], v[4][0],
                             v[4][1], v[4][0], v[4][2]))
                else:
                    if not fig:
                        exo.append('''$%s//%s$\\par et\\par $%s\perp%s$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                                 (v[4][0], v[4][1], v[4][0], v[4][2]))
                    cor.append('$%s//%s$\\par et\\par $%s\perp%s$ &' % 
                             (v[4][0], v[4][1], v[4][0], v[4][2]))
                if fig:
                    exo.append('& %s & & \\\\' % ('\n').join(figure(v[0],
                             v[1], v[2], v[3], v[4], v[5], lexos[i])))
                cor.append('%s & ' % ('\n').join(figure(v[0], v[1], v[2],
                         v[3], v[4], v[5], lexos[i])))
                cor.append(u'Si deux droites sont parallèles, alors toute perpendiculaire à l\'une est perpendiculaire à l\'autre. &')
                if v[5]:
                    cor.append('$(%s%s)\\perp(%s%s)$ \\\\\n  \\hline' % 
                             (v[4][2], v[4][3], v[4][0], v[4][2]))
                else:
                    cor.append('$%s\perp%s$ \\\\\n  \\hline' % (v[4][1],
                             v[4][2]))
            else:
                if v[5]:
                    if not fig:
                        exo.append('''$(%s%s)\\perp(%s%s)$\\par et\\par $(%s%s)\\perp(%s%s)$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                                 (v[4][0], v[4][1], v[4][0], v[4][2], v[4][2],
                                 v[4][3], v[4][0], v[4][2]))
                    cor.append('$(%s%s)\\perp(%s%s)$\\par et\\par $(%s%s)\\perp(%s%s)$ &' % 
                             (v[4][0], v[4][1], v[4][0], v[4][2], v[4][2],
                             v[4][3], v[4][0], v[4][2]))
                else:
                    if not fig:
                        exo.append('''$%s\\perp%s$\\par et\\par $%s\perp%s$ &
\\begin{pspicture*}[shift=-1.5](-2.1,-1.6)(2.1,1.6)
\\end{pspicture*}
& & \\\\''' % 
                                 (v[4][0], v[4][2], v[4][1], v[4][2]))
                    cor.append('$%s\\perp%s$\\par et\\par $%s\perp%s$ &' % 
                             (v[4][0], v[4][2], v[4][1], v[4][2]))
                if fig:
                    exo.append('& %s & & \\\\' % ('\n').join(figure(v[0],
                             v[1], v[2], v[3], v[4], v[5], lexos[i])))
                cor.append('%s &' % ('\n').join(figure(v[0], v[1], v[2],
                         v[3], v[4], v[5], lexos[i])))
                cor.append(u'Si deux droites sont perpendiculaires à une même troisième alors elles sont parallèles entre elles. &')
                if v[5]:
                    cor.append('$(%s%s)//(%s%s)$ \\\\\n  \\hline' % (v[4][0],
                             v[4][1], v[4][2], v[4][3]))
                else:
                    cor.append('$%s//%s$ \\\\\n  \\hline' % (v[4][0],
                             v[4][1]))
    exo.append('''\\hline
\\end{tabularx}
''')
    cor.append('\\end{tabularx}')


def _Proprietes():
    exo = ["\\exercice", u"Compléter le tableau suivant :\\par Les droites en gras sont parallèles.\\par """]
    cor = ["\\exercice*", u"Compléter le tableau suivant :\\par Les droites en gras sont parallèles.\\par """]

    enonce_prop(exo, cor)
    return (exo, cor)

class Proprietes(LegacyExercise):
    """Propriétés sur les droites"""

    tags = ["Sixième"]
    function = _Proprietes



from pyromaths.outils.decimaux import decimaux
from pyromaths.outils.Arithmetique import carrerise, pgcd, ppcm, factor
from math import sqrt
from .Fractions import Fraction

def produitfacteurs(facteurs):
    """Affiche sous forme de produit les éléments d'une liste."""
    prodfacteurs = ''
    for element in facteurs:
        prodfacteurs += str(element) + ' \\times '
    return prodfacteurs[:-7]

class Racine:
    def __init__(self, radicande, coeff=1, indice=2):
        if (radicande < 0) or not (isinstance(indice, int)):
            print("Erreur de définition ! Le radicande doit être positif et l'indice un nombre entier !")
        self.radicande = radicande
        self.indice = indice  # Nombre entier
        self.coeff = coeff  # Coeff devant la racine

    def __add__(self, other):
        if not (isinstance(other, Racine)):
            return str(self) + " + " + str(other)
        if (self.radicande == other.radicande) and (self.indice == other.indice):
            return [str(self.simplifie()[0]) + "+" + str(other.simplifie()[0]), Racine(self.radicande, self.coeff + other.coeff, self.indice)]
        else:
            return str(self) + " + " + str(other)

    def __radd__(self, other):
        return self +other

    def __sub__(self, other):
        if not (isinstance(other, Racine)):
            return str(self) + " + " + str(other)
        if (self.radicande == other.radicande) and (self.indice == other.indice):
            return Racine(self.radicande, self.coeff - other.coeff, self.indice)
        else:
            return (str(self) + " - " + str(other), "tset")

    def __rsub__(self, other):
        return -(self -other)

    def __mul__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, self.coeff * other, self.indice)
        elif self.indice == other.indice:
            return Racine(self.radicande * other.radicande, self.coeff * other.coeff, self.indice)
        else:
            return str(self) + ' x ' + str(other)

    def __rmul__(self, other):
        return self * other

    def __truediv__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, self.coeff // other, self.indice)
        elif self.indice == other.indice:
            return Racine(self.radicande / other.radicande, self.coeff / other.coeff, self.indice)
        else:
            return str(self) + ' / ' + str(other)

    def __rtruediv__(self, other):
        if (isinstance(other, float)) or (isinstance(other, int)):
            return Racine(self.radicande, other / self.coeff * self.radicande, self.indice)
        elif self.indice == other.indice:
            return Racine(other.radicande / self.radicande, other.coeff / self.coeff, self.indice)
        else:
            return str(other) + ' / ' + str(self)

    def __str__(self):
        if self.coeff == 0:
            return "0"
        else:
            if self.coeff == 1:
                coeff = ""
            else:
                coeff = decimaux(self.coeff, 1) + " \\, "
            if self.radicande == 0:
                coeff = ""
                racine = ""
                radicande = "0"
            elif self.indice == 2:
                racine = "\\sqrt{"
                radicande = decimaux(self.radicande, 1) + "}"
            else:
                racine = "\\sqrt[{0}]{{".format(self.indice)
                radicande = decimaux(self.radicande, 1) + "}"
            return coeff + racine + radicande

    def simplifie(self):
        facteurs = factor(self.radicande)
        prodfacteurs = produitfacteurs(facteurs)
        if self.indice == 2:
            racine = "\\sqrt{"
        else:
            racine = "\\sqrt[{0}]{{".format(self.indice)
        coeff = self.coeff
        radicande = self.radicande
        detail = [str(self), ' = ', str(coeff) + racine + prodfacteurs + '}']
        for element in facteurs:
            if facteurs.count(element) % self.indice == 0:
                coeff *= element
                for dummy in range(self.indice):
                    facteurs.remove(element)
                radicande = radicande // (element ** (self.indice))
                prodfacteurs = produitfacteurs(facteurs)
                detail.append(' = ')
                detail.append(str(coeff) + ' \\times ' + racine + prodfacteurs + '}')
        if radicande == 1:
            return (coeff, detail)
        else:
            return (Racine(radicande, coeff, self.indice), detail)

def simplifie_racine(n):
    '''renvoie coeff,radicande où sqrt(n)=coeff*sqrt(radicande)'''
    if n == 0:
        return 0, 0
    else:
        ncar = carrerise(n)
        return int(sqrt(n // ncar)), ncar


def pTeX(n):
    """renvoie (n) si n<0"""
    if n < 0:
        return "(" + decimaux(n) + ")"
    else:
        return decimaux(n)

def tTeX(n):
    if n == 1:
        return ""
    elif n == -1:
        return "-"
    elif n >= 0:
        return "+" + decimaux(n)
    else:
        return decimaux(n)



class RacineDegre2:
    def __init__(self, numerateur=0, denominateur=1, coeff=1, radicande=0):
        """Constructeur de la forme (a+c*racine(d))/b"""
        self.numerateur = numerateur  # a
        self.denominateur = denominateur  # b
        self.coeff = coeff  # c
        self.radicande = radicande  # d

    def __str__(self):
        if isinstance(self.numerateur, str):
            # utilisé dans le détail de la simplification
            numerateur = self.numerateur
        else:
            numerateur = decimaux(self.numerateur) * (self.numerateur != 0)  # renvoie "" si self.numerateur=0
        if self.radicande != 0:
            if isinstance(self.coeff, str):  # utilisé dans le détail de la simplification
                if self.coeff[0] != "-" and self.coeff[0] != "+":
                    numerateur += "+"
                numerateur += self.coeff + "\\sqrt{" + decimaux(self.radicande) + "}"
            elif self.coeff == 1:
                numerateur += "+\\sqrt{" + decimaux(self.radicande) + "}"
            elif self.coeff == -1:
                numerateur += "-\\sqrt{" + decimaux(self.radicande) + "}"
            else:
                numerateur += tTeX(self.coeff) + "\\sqrt{" + decimaux(self.radicande) + "}"
        if numerateur == '':
            numerateur = '0'
        if self.denominateur == 1:
            result = numerateur
        else:
            result = "\\dfrac{%s}{%s}" % (numerateur, self.denominateur)

        if result[0] == '+':
            return result[1:]
        else:
            return result

    def simplifie(self, detail=False):
        liste_detail = []
        coeff, radicande = simplifie_racine(self.radicande)
        numerateur = self.numerateur
        if self.radicande != 0:
            if self.coeff == 1:
                    det_coeff = "+ "
            elif self.coeff == -1:
                det_coeff = "- "
            else:
                det_coeff = "%s\\times " % (self.coeff)
            if coeff != 1 or radicande == 1:
                det_coeff += str(coeff)
        else:
            det_coeff = "0"
        if radicande == 1:
            # det_coeff="%s\\times%s"%(tTeX(self.coeff),coeff)
            liste_detail.append("\\dfrac{%s %s}{%s}" % \
                                    (self.numerateur, det_coeff , self.denominateur))
            radicande = 0
            numerateur = self.numerateur + (self.coeff) * int(coeff)
            coeff = 0

        if coeff != 1:
            liste_detail.append(str(RacineDegre2(numerateur,
                                                 self.denominateur,
                                                 det_coeff,
                                                 radicande)))

        coeff = (self.coeff) * int(coeff)

        simplifie = pgcd(pgcd(coeff, numerateur), self.denominateur)
        numerateur = numerateur // simplifie
        coeff = coeff // simplifie
        denominateur = self.denominateur // simplifie
        if simplifie != 1:
            if radicande != 0 or denominateur != 1:
                det_numerateur = "%s_{\\times %s}" % (numerateur, pTeX(simplifie))
                det_denominateur = "%s_{\\times %s}" % (denominateur, pTeX(simplifie))
                det_coeff = "%s_{\\times %s}" % (coeff, pTeX(simplifie))
                liste_detail.append(str(RacineDegre2(det_numerateur, det_denominateur, det_coeff, radicande)))
            liste_detail.append(str(RacineDegre2(numerateur, denominateur, coeff, radicande)))
        if detail:
            return RacineDegre2(numerateur, denominateur, coeff, radicande), liste_detail
        return RacineDegre2(numerateur, denominateur, coeff, radicande)

    def __add__(self, other):
        if isinstance(other, RacineDegre2):
            if self.radicande == other.radicande or self.radicande == 0 or other.radicande == 0:
                radicande = max(self.radicande, other.radicande)
                premier, second = self, other
            else:
                premier = self.simplifie()
                second = other.simplifie()
                if self.radicande == other.radicande or self.radicande == 0 or other.radicande == 0:
                    radicande = max(self.radicande, other.radicande)
                else:
                    return NotImplemented

            denominateur = ppcm(premier.denominateur, second.denominateur)
            facteur1 = denominateur // premier.denominateur
            facteur2 = denominateur // second.denominateur
            # if self.radicande==other.radicande:
            coeff = premier.coeff * facteur1 * (premier.radicande != 0) + second.coeff * facteur2 * (second.radicande != 0)
            if coeff == 0:
                radicande = 0
            return RacineDegre2(premier.numerateur * facteur1 + second.numerateur * facteur2,
                                denominateur,
                                coeff,
                                radicande)
        elif isinstance(other, int):
            return self +RacineDegre2(other)
        elif isinstance(other, Fraction):
            return self +RacineDegre2(other.numerateur, other.denominateur)

    def __radd__(self, other):
        return self +other
    def __neg__(self):
        return RacineDegre2(-self.numerateur, self.denominateur, -self.coeff, self.radicande)

    def __sub__(self, other):
        return self +(-other)

    def __rsub__(self, other):
        return -self + other

    def __mul__(self, other):
        if isinstance(other, Fraction):
            return self * RacineDegre2(other.numerateur, other.denominateur, 0, self.radicande)
        elif isinstance(other, int):
            return self * RacineDegre2(other, 1, 0, self.radicande)
        else:
            radicande = max(self.radicande, other.radicande)  # cela autorise d'avoir un radicande=0 mais n'efface pas l'autre
            coeff = self.numerateur * (other.coeff * (other.radicande != 0)) + (self.coeff * (self.radicande != 0)) * other.numerateur
            if coeff == 0:
                radicande = 0
            numerateur = self.numerateur * other.numerateur + (self.coeff * other.coeff) * (not(self.radicande == 0 or other.radicande == 0)) * self.radicande
            return RacineDegre2(numerateur,
                            self.denominateur * other.denominateur,
                            coeff,
                            radicande)
    def __invert__(self):
        return self.denominateur * RacineDegre2(self.numerateur, self.numerateur ** 2 - self.coeff ** 2 * self.radicande, -self.coeff, self.radicande)
    def __truediv__(self, other):
        return self * ~other
    def __rtruediv__(self, other):
        return ~self * other
    def __rmul__(self, other):
        return self * other
    def __pow__(self, n):
        result = 1
        for dummy in range(n):
            result = result * self
        return result
    def __float__(self):
        return (self.numerateur + self.coeff * sqrt(self.radicande)) / self.denominateur
    def __lt__(self, other):
        if float(self) - float(other) < 0: return True
        else: return False
    def __eq__(self, other):
        return float(self) == float(other)
    def __ne__(self, other):
        return not self == other
    def __gt__(self, other):
        if float(self) - float(other) > 0: return True
        else: return False
    def __ge__(self, other):
        if float(self) - float(other) >= 0: return True
        else: return False
    def __le__(self, other):
        if float(self) - float(other) <= 0: return True
        else: return False



#!/usr/bin/env python3
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
# from pyromaths.outils import ecrit_tex, valeur_alea, signe, pgcd, ppcm

import random
from decimal import Decimal
from pyromaths.ex import LegacyExercise

def valeurs(nb, entier=1):  # renvoie les 2 listes contenant les opérateurs et les opérandes.
    listoperateurs = [
        '+',
        '*',
        '-',
        '/',
        '(',
        '(',
        '(',
        '(',
        ')',
        ')',
        ')',
        ')',
        ]
    loperateurs = []
    loperandes = []
    i = 0  # nombre d'opérateurs créés
    p = 0  # nombre de parenthèses ouvertes
    cpt = 0  # compteur pour éviter que le programme ne boucle.
    while i < nb - 1:
        cpt = cpt + 1
        if cpt > 10:  # On recommence
            (cpt, i, p, loperateurs) = (0, 0, 0, [])
        if p:
            if loperateurs[-1] == '(':  # On n'écrit pas 2 parenthèses à suivre
                operateur = listoperateurs[random.randrange(4)]
            else:
                operateur = listoperateurs[random.randrange(12)]
        elif loperateurs == []:

            # On ne commence pas par une parenthèse

            operateur = listoperateurs[random.randrange(4)]
        else:
            operateur = listoperateurs[random.randrange(8)]
        if nb > 3:
            test = ('-*/').find(operateur) >= 0 and \
                   loperateurs.count(operateur) < 1 or \
                   operateur == '+' and \
                   loperateurs.count(operateur) < 2
        else:
            test = ('-*/+').find(operateur) >= 0 and \
                   loperateurs.count(operateur) < 1
        if test:

            # On n'accepte pas plus de 1 produit, différence, quotient et de 2
            # sommes ou parenthèses par calcul.

            if i == 0 or loperateurs[-1] != '(' or ('*/').find(operateur) < \
                0:  # pas de * ou / dans une parenthèse.
                i = i + 1
                loperateurs.append(operateur)
        elif operateur == '(' and (')+').find(loperateurs[-1]) < 0:

            # Il ne peut y avoir de ( après une ) ou après un +

            p = p + 1
            loperateurs.append(operateur)
        elif operateur == ')':
            p = p - 1
            loperateurs.append(operateur)
    while p > 0:
        loperateurs.append(')')
        p = p - 1
    if entier:
        loperandes = [Decimal(random.randrange(12) + 2) for i in range(nb)]
    else:
        loperandes = [(Decimal(random.randrange(88) + 12))  / Decimal(10) for i in range(nb)]
    return (loperateurs, loperandes)


#######################


def affichage(loperateurs, loperandes):
    j = 0  # compteur des operateurs
    calcul = '%s' % nb_decimal((loperandes[0],))
    for i in range(len(loperandes) - 1):
        if loperateurs[j] == ')':  # Il reste une opération mais je ferme d'abord une ou plusieurs parenthèses
            cpt = 1
            while loperateurs[j + cpt] == ')':
                cpt = cpt + 1  # j+cpt est la position de l'opération qui suit
        else:
            cpt = 0
        if j + cpt < len(loperateurs) - 1:  # Il reste au moins une opération, donc peut-etre une parenthèse ouvrante
            while loperateurs[j + cpt + 1] == '(':  # j+cpt est la position de la dernière parenthèse (
                cpt = cpt + 1
        for k in range(cpt + 1):
            calcul = calcul + '%s' % loperateurs[j + k]
        calcul = calcul + '%s' % nb_decimal((loperandes[i + 1],))
        j = j + cpt + 1
    while j < len(loperateurs):
        calcul = calcul + '%s' % loperateurs[j]
        j = j + 1
    calcul = ('\\times ').join(calcul.split('*', 2))
    calcul = ('\\div ').join(calcul.split('/', 2))
    return calcul


def nb_decimal(a):  # verifie si des nombres décimaux dans le tuple a sont en fait des nombres entiers et change leur type
    # liste = []
    liste = [('{,}').join(str(Decimal(e)).split('.',2)) for e in a]
    # for i in range(len(a))
    #     if str(a[i]).endswith('.0'):
    #         liste.append(int(a[i] + .1))
    #     else:
    #         liste.append(('{,}').join(str(a[i]).split('.', 2)))
    return tuple(liste)


def verifie_calcul(listoperateurs, listoperandes, entier=1):

    # Vérifie que l'opération proposée est réalisable sans division
    # décimale ni nombre négatif

    p = 0
    loperateurs = listoperateurs[-1]
    loperandes = listoperandes[-1]
    if len(loperandes) == 1:
        return (listoperateurs, listoperandes)
    else:
        nbpar = loperateurs.count('(')
        nbmul = loperateurs.count('*')
        nbdiv = loperateurs.count('/')
        if nbpar:
            index = -1
            while p < nbpar:
                index = loperateurs[index + 1:].index('(') + index + 1
                p = p + 1
                if p < nbpar and loperateurs[index + 1:].index('(') > \
                    loperateurs[index + 1:].index(')'):
                    nbpar = p
            if loperateurs[index + 2] == ')':
                a = calcul(loperandes[(index + 1) - nbpar], loperateurs[index +
                           1], loperandes[(index + 2) - nbpar], entier)
                if a != 'hp':
                    al = loperateurs[:index]
                    al.extend(loperateurs[index + 3:])
                    loperateurs = al
                    al = loperandes[:(index + 1) - nbpar]
                    al.append(a)
                    al.extend(loperandes[index - nbpar + 3:])
                    loperandes = al
                    listoperateurs.append(loperateurs)
                    listoperandes.append(loperandes)
                    return verifie_calcul(listoperateurs, listoperandes)
        elif nbmul or nbdiv:
            (indexm, indexd) = (100, 100)
            if nbmul:
                indexm = loperateurs.index('*')
            if nbdiv:
                indexd = loperateurs.index('/')
            index = min(indexm, indexd)
            a = calcul(loperandes[index], loperateurs[index], loperandes[index +
                       1], entier)
            if a != 'hp':
                al = loperateurs[:index]
                al.extend(loperateurs[index + 1:])
                loperateurs = al
                al = loperandes[:index]
                al.append(a)
                al.extend(loperandes[index + 2:])
                loperandes = al
                listoperateurs.append(loperateurs)
                listoperandes.append(loperandes)
                return verifie_calcul(listoperateurs, listoperandes)
        else:
            a = calcul(loperandes[0], loperateurs[0], loperandes[1],
                       entier)
            if a != 'hp':
                loperateurs = loperateurs[1:]
                al = [a]
                al.extend(loperandes[2:])
                loperandes = al
                listoperateurs.append(loperateurs)
                listoperandes.append(loperandes)
                return verifie_calcul(listoperateurs, listoperandes)


def calcul(a, op, b, entier=1):  # retourne 'hp' (hors programme) ou le résultat de l'opération
    if op == '+':
        return a + b
    elif op == '*':
        return a * b
    elif op == '-':
        if a > b:
            return a - b
        else:
            return 'hp'
    else:
        if (a * 100) % (b * 100) and entier:
            return 'hp'
        elif not entier and (a * 1000) % (b * 100):
            return 'hp'
        else:
            return a // b


def _priorites():
    nb = 9  # nombre de calculs
    exo = ["\\exercice",
           u"Calculer les expressions suivantes en détaillant les calculs.\n",
           "\\begin{multicols}{3}", "\\noindent"]
    cor = ["\\exercice*",
           u"Calculer les expressions suivantes en détaillant les calculs.",
           "\\begin{multicols}{3}", "\\noindent"]
    i = 0
    while i < nb:
        if i < 3:  # Les 3 premiers calculs ne comportent que 3 nombres
            (loperateurs, loperandes) = valeurs(3)
        elif i > 6:
            (loperateurs, loperandes) = valeurs(5, 0)  # Les derniers calculs comportent des nombres décimaux
        else:
            (loperateurs, loperandes) = valeurs(6)

        if i > 6:
            liste = verifie_calcul([loperateurs], [loperandes], entier=0)
        else:
            liste = verifie_calcul([loperateurs], [loperandes], entier=1)
        if liste != None:
            i = i + 1
            for j in range(len(liste[0])):
                if j == 0:
                    exo.append("\\[ \\thenocalcul = %s \\]" %
                               affichage(liste[0][j], liste[1][j]))
                    exo.append('\\stepcounter{nocalcul}%')
                if j == len(liste[0]) - 1:
                    cor.append("\\[ \\boxed{\\thenocalcul = %s} \\]" %
                               affichage(liste[0][j], liste[1][j]))
                    cor.append('\\stepcounter{nocalcul}%')
                else:
                    cor.append("\\[ \\thenocalcul = %s \\]" %
                               affichage(liste[0][j], liste[1][j]))
    exo.append("\\end{multicols}")
    cor.append("\\end{multicols}")
    return (exo, cor)

class priorites(LegacyExercise):
    """Priorités opératoires"""

    tags = ["Cinquième"]
    function = _priorites



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randrange

def genere_points(xmin, xmax, ymin, ymax, simple=False):

    if simple:  # génère 3 intervalles de variations /\/ ou \/\
        if randrange(1):
            variation = [0, 1, 0]
        else:
            variation = [1, 0, 1]
        longueur = xmax - xmin
        hauteur = ymax - ymin
        x_variation = [xmin]
        x_variation.append(x_variation[0] + randrange(int(0.25 * longueur), int(0.4 * longueur)))
        x_variation.append(x_variation[1] + randrange(int(0.25 * longueur), int(0.4 * longueur)))
        x_variation.append(xmax)
        liste_points = [(x_variation[0], randrange(ymin + (1 - variation[0]) * int(0.4 * hauteur), ymax - variation[0] * int(0.4 * hauteur)))]
        for i in [1, 2, 3]:
            if variation[i - 1]:
                liste_points.append((x_variation[i], randrange(liste_points[i - 1][1], ymax)))
            else:
                liste_points.append((x_variation[i], randrange(ymin, liste_points[i - 1][1])))
    else:
        liste_points = [(xmin, 0)]
        for i in range(10):
            liste_points.append((liste_points[i][0] + randrange(2, 5), randrange(ymin, ymax)))
    return liste_points

def bezier(simple=False, xmin=-15, xmax=15, ymin=-12, ymax=12):
    liste_points = genere_points(simple=simple, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
    variation = []
    largeur = liste_points[-1][0] - liste_points[0][0]
    points = [liste_points[0]]
    for i in range(1, len(liste_points)):
        if liste_points[i - 1][1] < liste_points[i][1]:
            points.append(liste_points[i])
        elif liste_points[i - 1][1] > liste_points[i][1]:
            points.append(liste_points[i])
    deltaxx = 0.3 * (points[1][0] - points[0][0])
    nderiv = (points[1][1] - points[0][1]) / (points[1][0] - points[0][0])
    apres = [(points[0][0] + 0.5, points[0][1] + deltaxx * nderiv)]
    avant = []
    if points[0][1] < points[1][1]:
        variation += '+'
    else:
        variation += '-'
    x_variation = [points[0][0]]
    y_variation = [points[0][1]]

    for i in range(1, len(points) - 1):
        if (points[i][1] <= points[i - 1][1] and points[i][1] <= points[i + 1][1]):
            nderiv = 0
            x_variation.append(points[i][0])
            y_variation.append(points[i][1])
            variation += '+'
        elif (points[i][1] >= points[i - 1][1] and points[i][1] >= points[i + 1][1]):
            nderiv = 0
            x_variation.append(points[i][0])
            y_variation.append(points[i][1])
            variation += '-'
        else:
            nderiv = (points[i - 1][1] - points[i + 1][1]) / (points[i - 1][0] - points[i + 1][0])
        deltax = 0.3 * (points[i - 1][0] - points[i][0])
        deltaxx = 0.3 * (points[i + 1][0] - points[i][0])
        avant.append((points[i][0] + deltax, points[i][1] + deltax * nderiv))
        apres.append((points[i][0] + deltaxx, points[i][1] + deltaxx * nderiv))
    avant.append((points[len(points) - 1][0] - 0.5, points[len(points) - 1][1] - 0))
    string = str(points[0])
    for i in range(len(points) - 1):
        string += str(apres[i]) + str(avant[i]) + str(points[i + 1])
    textbezier = "\\input{tabvar}\n\
                \psset{unit=" + str(14 / largeur) + "cm}\n\
                \\begin{pspicture}"
    textbezier += "(" + str(xmin) + "," + str(ymin) + ")(" + str(liste_points[-1][0]) + "," + str(ymax) + ")\n"
    textbezier += "\psgrid[subgriddiv=0,griddots=10,gridlabels=0pt]\n\
                \psaxes[linewidth=1pt,%\n\
                ticks=none,%\n\
                %labels=none\n\
                Dx=2\n\
                ]{->}(0,0)"
    textbezier += "(" + str(xmin) + "," + str(ymin) + ")(" + str(liste_points[-1][0]) + "," + str(ymax) + ")\n"
    impression = textbezier + "\psbezier" + string + "\n" + "\\end{pspicture}\n\\vspace{2cm}\n\n"
    impression += "\\begin{tikzpicture}\n\\tkzTabInit[espcl=1]\n{$x$ / .5,$f'\\,(x)$ / .5}\n"
    impression += "{$" + "$,$".join([str(i) for i in x_variation]) + "$}\n"
    impression += "\\tkzTabLine{," + ",".join(y_variation) + "}\n\end{tikzpicture}\n"
#===============================================================================
# {$x$ / 1,$g(x)$ / 1}%
# {$0$,$1$,$2$}%
# \tkzTabLine{d,+,0,-,d}
# \end{tikzpicture}"
#===============================================================================
    impression += "$$\n\
                \\tabvar{%\n\
                \\tx{x}&\\tx{" + str(x_variation[0]) + "}"
    for i in range(1, len(x_variation)):
        impression += "&& \\tx{" + str(x_variation[i]) + "}"
    impression += "&& \\tx{" + str(points[-1][0]) + "}\\cr\n"
    impression += "\\tx{f(x)}"
    for i in range(len(variation)):
        if variation[i] == '+':
            impression += "&\\txb{" + str(y_variation[i]) + "}&\\fm"
        else:
            impression += "&\\txh{" + str(y_variation[i]) + "}&\\fd"
    if variation[-1] == '+':
        impression += "&\\txh{" + str(points[-1][1]) + "}"
    else:
        impression += "&\\txb{" + str(points[-1][1]) + "}"
    return impression + "\\cr\n}$$\n\n"


def tab_var(fonc, Intervalle=["-\\infty", "+\\infty"]):
    from pyromaths.classes.Polynome import Polynome
    if isinstance(fonc, Polynome):
        return tab_var_poly(fonc, Intervalle[0], Intervalle[1])
    else:
        return _("Je ne sais pas étudier les variations de cette fonction.")

def tab_var_poly(P, borneinf="-\\infty" , bornesup=u"+\\infty"):
    Intervalle = [borneinf, bornesup]
    # print"borneinf=",borneinf
    tab_var = "\\input{tabvar}\n"
    tab_var += "$$\\tabvar{%\n"
    if P.deg == 0:
        tab_var += tabvar_x(Intervalle, P.var)
        Cste = str(P(0))
        tab_var += "\\tx{f(" + P.var + ")}&\\tx{" + Cste + "}&\\fhm&\\tx{" + Cste + "}\\cr\n"
        return tab_var + "}$$\n"
    elif P.deg == 1:
        tab_var += tabvar_x(Intervalle, P.var)
        if P.dictio[1] > 0:
            tab_var += "\\tx{f(" + P.var + ")}& &\\fm& \\cr\n"
        else:
            tab_var += "\\tx{f(" + P.var + ")}& &\\fd&\\cr\n"
        return tab_var + "}$$\n"
    elif P.deg == 2:
        alpha = -P.dictio[1] //  (2 * P.dictio[2])
        extremum = P(alpha)
        if (borneinf == "-\\infty" or Intervalle[0] < alpha) and\
           (Intervalle[1] == "+\\infty" or Intervalle[1] > alpha):
            listex = [Intervalle[0], alpha, Intervalle[1]]
            if P.dictio[0] > 0:
                tab_var_y = "\\tx{f(" + P.var + ")}& &\\fd&\\txb{" + nombre_TeX(extremum) + "}&\\fm&\\cr\n"
            else:
                tab_var_y = "\\tx{f(" + P.var + ")}& &\\fm&\\txh{" + nombre_TeX(extremum) + "}&\\fd&\\cr\n"
        else:
            listex = Intervalle
            if P.dictio > 0:
                if not(borneinf == "-\\infty") and Intervalle[0] > alpha:
                    tab_var_y = "\\tx{f(" + P.var + ")}& &\\fm& \\cr\n"
                else:
                    tab_var_y = "\\tx{f(" + P.var + ")}& &\\fd&\\cr\n"
            else:
                if not(borneinf == "-\\infty") and Intervalle[0] > alpha:
                    tab_var_y = "\\tx{f(" + P.var + ")}& &\\fd&\\cr\n"
                else:
                    tab_var_y = "\\tx{f(" + P.var + ")}& &\\fm& \\cr\n"
        tab_var += tabvar_x(listex, P.var)
        tab_var += tab_var_y
        return tab_var + "}$$\n"
    elif P.deg == 3:
        return _(u"attends un peu pour le degré 3")
        # #Les fonctions développées pour les équations du second degré doivent aider
    else:
        return _(u"Degré trop élevé")
            # #Cas particulier où P' a une racine évidente, ou polynome bicarré


def tabvar_x(listex, var):
    '''imprime la premiere ligne du tabvar'''
    var_x = "\\tx{" + var + "}&\\tx{" + nombre_TeX(listex[0]) + "}"
    for i in range(1, len(listex)):
        var_x += "&&\\tx{" + nombre_TeX(listex[i]) + "}"
    return var_x + "\\cr\n"

def nombre_TeX(nombre):  # A remplacer par sepmillier
    from pyromaths.classes.Fractions import Fraction
    if type(nombre) in [type(int), type(float)]:
        return str(nombre)
    elif isinstance(nombre, Fraction):
        if nombre.denominateur == 1:
            return nombre_TeX(nombre.numerateur)
        else:
            return "\\dfrac{" + str(nombre.numerateur) + "}{" + str(nombre.denominateur) + "}"
    else:
        return str(nombre)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from pyromaths import ex
from random import randrange, shuffle
#===============================================================================
# from pyromaths.outils import Priorites3
# from pyromaths.classes.Fractions import Fraction
# from pyromaths.classes.PolynomesCollege import Polynome, factoriser
# from pyromaths.classes.SquareRoot import SquareRoot
from pyromaths.outils.Arithmetique import valeur_alea
from pyromaths.outils.Affichage import decimaux
#===============================================================================

def PointsCourbes(sens_var=(-1, -1, 0, 1, 1), nb_var=(3, 5), absc=(-5, 5), ordo=(-4, 4), cstes=(0, 1)):
    """Définit les ordonnées entières dans [-5 ; 5] de points d'abscisses entières dans [-5 ; 5]
    de telle sorte que la fonction sur chaque intervalle comporte entre 3 et 5 variations différentes,
    une seule section pouvant être constante.
    La fonction doit avoir des images positives et négatives.
    variations= [[-1, -6, -2], [1, -2, 1], [-1, 1, 2], [0, 2, 4]]
    """
    while True:
        lY, sens, variations = [valeur_alea(ordo[0], ordo[1])], [], []
        nb_cste = 0
        for dummy in range(absc[1] - absc[0]):
            while True:
                lg = valeur_alea(1, 4)
                # pente maximale entre deux points consécutifs
                sens.append(sens_var[randrange(len(sens_var))])
                nY = lY[-1] + lg * sens[-1]
                if ordo[0] <= nY <= ordo[1]:
                    if nY * lY[-1] < 0: lY.append(0)
                    else: lY.append(nY)
                    if not variations or sens[-1] != variations[-1][0]:
                        variations.append([sens[-1], dummy + absc[0], dummy + absc[0] + 1])
                    else:
                        variations[-1][2] = dummy + absc[0] + 1
                    break
                else:
                    sens.pop(-1)
        if nb_var[0] <= len(variations) <= nb_var[1]:
            extremums = []
            gextr = [100, -100]  # extrema sur l'esnble de définition
            for i in range(len(variations)):
                if lY[variations[i][1] - absc[0]] > gextr[1]: gextr[1] = lY[variations[i][1] - absc[0]]
                elif lY[variations[i][1] - absc[0]] < gextr[0]: gextr[0] = lY[variations[i][1] - absc[0]]
                if extremums.count(lY[variations[i][1] - absc[0]]) > 0 and variations[i - 1][0] != 0:
                    nb_cste = cstes[1] + 1
                else:
                    extremums.append(lY[variations[i][1] - absc[0]])
                if variations[i][0] == 0:nb_cste += 1
            if extremums.count(lY[-1]): nb_cste = cstes[1] + 1
            if lY[-1] > gextr[1]: gextr[1] = lY[-1]
            elif lY[-1] < gextr[0]: gextr[0] = lY[-1]
            if gextr[0] * gextr[1] >= 0: nb_cste = cstes[1] + 1
            if cstes[0] <= nb_cste <= cstes[1]: break
    variations[-1][2] = absc[1]
    return lY, variations

def remplissage_tableau(lX, lY, variations):
    """Renvoie la liste des abscisses, des ordonnées et des zéros (position et abscisse)
    pour compléter un tableau de variations"""
    lAbsc = [str(v[1]) for v in variations] + [str(variations[-1][2])]
    lOrdo = []
    for index in range(len(variations)):
        lOrdo.append(lY[variations[index][1] - lX[0]])
    lOrdo.append(lY[-1])
    lZeros = []
    for absc in range(lX[0], lX[-1] + 1):
        if lY[absc - lX[0]] == 0:
            for index in range(len(variations)):
                if variations[index][1] < absc < variations[index][2]:
                    if variations[index][0] != 0:
                        lZeros.append([index + 1, index + 2, absc])
                    break
                if variations[index][1] == absc or variations[index][2] == absc:
                    break
    return lAbsc, lOrdo, lZeros

def integration_zeros(lX, lY, variations):
    """Renvoie la liste des abscisses et des ordonnées des extrema locaux et des zéros"""
    lAbs, lOrd, lZeros = remplissage_tableau(lX, lY, variations)
    lZeros.reverse()
    for z in lZeros:
        lAbs.insert(z[0], z[2])
        lOrd.insert(z[0], 0)
    return lAbs, lOrd


class Vf1SensEtTableau(ex.TexExercise):
    """Sens et Tableau de variations"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        lX = [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5]
        self.lX = lX
        self.lY1, self.variations1 = PointsCourbes(sens_var=(-1, 1))
        self.lY2, self.variations2 = PointsCourbes()



    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Quel est le sens de variation de la fonction $f$ ? Répondre par une phrase en précisant les intervalles.'))
        exo.append(_(r'\item Tracer les tableaux de variation des fonctions $f$ et $g$.'))
        exo.append(r'\end{enumerate}')
        exo.append(r'\begin{center}')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in self.lY1])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_f$", (%s,%s), align=%s);' % (self.lX[0], self.lY1[0], ('SE', 'NW', 'SW')[self.variations1[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\kern1cm')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in self.lY2])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_g$", (%s,%s), align=%s);' % (self.lX[0], self.lY2[0], ('SE', 'NW', 'SW')[self.variations2[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\end{center}')

        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        ans = _(u'\\item la fonction $f$ est')
        dvar = {_(u'décroissante'): [], _(u'croissante'): [], _(u'constante'):[]}
        for v in self.variations1:
            if v[0] == 1: dvar[_(u'croissante')].append('$\interval{%s}{%s}$' % (v[1], v[2]))
            elif v[0] == -1: dvar[_(u'décroissante')].append('$\interval{%s}{%s}$' % (v[1], v[2]))
            elif v[0] == 0: dvar[_(u'constante')].append('$\interval{%s}{%s}$' % (v[1], v[2]))
            else: raise ValueError(_('variation non attendue'))
        if dvar[_(u'décroissante')]: ans += u' décroissante sur %s,' % ' et '.join(dvar[_(u'décroissante')])
        if dvar[_(u'croissante')]: ans += u' croissante sur %s' % ' et '.join(dvar[_(u'croissante')])
        if dvar[_(u'constante')]: ans += u' constante sur %s' % ' et '.join(dvar[_(u'constante')])
        ans += '.'
        exo.append(ans)
        exo.append(r'\item')
        exo.append(r'\begin{tabular}[t]{ll}')
        for j in range(2):
            variations = [self.variations1, self.variations2][j]
            lY = [self.lY1, self.lY2][j]
            exo.append(r'\begin{tikzpicture}[scale=1]')
            exo.append(r'\tkzTabInit[lgt=1.2,espcl=1.2]{$x$/1,$%s\,(x)$/3}{$%s$}' % (['f', 'g'][j], '$,$'.join([str(v[1]) for v in variations] + [str(variations[-1][2])])))
            ans = r'\tkzTabVar{'  # %-/$-2$,+/$5$,-/$-3$,+/$-1$,-/$-4$}'
            trois_niv = False
            for index in range(len(variations)):
                yindex = variations[index][1] + 5
                if variations[index][0] == 1:
                    if trois_niv:
                        ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                        trois_niv = False
                    else: ans += r'-/$%s$,' % lY[yindex]
                if variations[index][0] == -1:
                    if trois_niv:
                        ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                        trois_niv = False
                    else: ans += r'+/$%s$,' % lY[yindex]
                if variations[index][0] == 0:
                    if index > 0 and index < len(variations) - 1 and variations[index - 1][0] * variations[index + 1][0] > 0:
                        trois_niv = True
                    if trois_niv: ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    elif index > 0:
                        if variations[index - 1][0] == 1: ans += r'+/$%s$,' % lY[yindex]
                        else: ans += r'-/$%s$,' % lY[yindex]
                    elif index < len(variations) - 1:
                        if variations[index + 1][0] == 1: ans += r'-/$%s$,' % lY[yindex]
                        else: ans += r'+/$%s$,' % lY[yindex]
            if variations[-1][0] == 1:  ans += r'+/$%s$}' % lY[-1]
            elif variations[-1][0] == -1:  ans += r'-/$%s$}' % lY[-1]
            elif variations[-1][0] == 0:
                if variations[-2][0] == 1:  ans += r'+/$%s$}' % lY[-1]
                elif variations[-2][0] == -1:  ans += r'-/$%s$}' % lY[-1]
            exo.append(ans)
            # Place les zéros
            for absc in range(-5, 6):
                if lY[absc + 5] == 0:
                    for index in range(len(variations)):
                        if variations[index][1] < absc < variations[index][2]:
                            if variations[index][0] != 0:
                                exo.append(r'\tkzTabVal{%s}{%s}{0.5}{\scriptsize $%s$}{$0$}' % (index + 1, index + 2, absc))
                            break
                        if variations[index][1] == absc or variations[index][2] == absc:
                            break
            exo.append(r'\end{tikzpicture}')
            exo.append(r'&')
        exo.pop(-1)
        exo.append(r'\end{tabular}')
        exo.append(r'\end{enumerate}')

        return "\n".join(exo)

def extrema(lX, lY, intervalle):
    extremums = [[None, 1000], [None, -1000]]
    for i in range(intervalle[0] - lX[0], intervalle[1] + 1 - lX[0]):
        if lY[i] < extremums[0][1]:
            extremums[0][0] = lX[i]
            extremums[0][1] = lY[i]
        if lY[i] > extremums[1][1]:
            extremums[1][0] = lX[i]
            extremums[1][1] = lY[i]
    return extremums

class Vf2ExtremaGraphiques(ex.TexExercise):
    """Extrema et représentation graphique"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        fin = False
        while not fin:  # Évite les boucles infinies
            lX = [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5]
            self.lX = list(lX)
            self.lY1, self.variations1 = PointsCourbes(sens_var=(-1, 1))
            self.lY2, self.variations2 = PointsCourbes()
            self.extremum = (u'maximum', u'minimum')[randrange(2)]
            for dummy in range(20):
                lX = list(self.lX)
                intervalle1 = [lX.pop(randrange(len(lX)))]
                intervalle1.append(lX.pop(randrange(len(lX))))
                intervalle1 = sorted(intervalle1)
                [gmin, gmax] = extrema(self.lX, self.lY1, [-5, 5])
                [lmin, lmax] = extrema(self.lX, self.lY1, intervalle1)
                if abs(lmin[0] - lmax[0]) > 1 and lmin[1] != gmin[1] and lmax[1] != gmax[1]:
                    fin = True
                    break
            if fin:
                fin = False
                for dummy in range(20):
                    lX = list(self.lX)
                    lX.remove(intervalle1[0])
                    lX.remove(intervalle1[1])
                    intervalle2 = [lX.pop(randrange(len(lX)))]
                    intervalle2.append(lX.pop(randrange(len(lX))))
                    intervalle2 = sorted(intervalle2)
                    [gmin, gmax] = extrema(self.lX, self.lY1, [-5, 5])
                    [lmin, lmax] = extrema(self.lX, self.lY1, intervalle2)
                    if abs(lmin[0] - lmax[0]) > 1 and lmin[1] != gmin[1] and lmax[1] != gmax[1]:
                        fin = True
                        break
        self.intervalle1 = intervalle1
        self.intervalle2 = intervalle2

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{multicols}{2}')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Quels sont les extrema de la fonction $f$ ?'))
        exo.append(_(u'\\item Quel est le %s de $f$ sur l\'intervalle $\interval{%s}{%s}$ ?') % (self.extremum, self.intervalle1[0], self.intervalle1[1]))
        exo.append(_(u'\\item Quels sont les extrema de la fonction $g$ ?'))
        exo.append(_(u'\\item Quels sont les extrema de $g$ sur l\'intervalle $\interval{%s}{%s}$ ?') % (self.intervalle2[0], self.intervalle2[1]))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{multicols}')
        exo.append(r'\begin{center}')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in self.lY1])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_f$", (%s,%s), align=%s);' % (self.lX[0], self.lY1[0], ('SE', 'NW', 'SW')[self.variations1[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\kern1cm')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in self.lY2])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_g$", (%s,%s), align=%s);' % (self.lX[0], self.lY2[0], ('SE', 'NW', 'SW')[self.variations2[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\end{center}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        extr = extrema(self.lX, self.lY1, [self.lX[0], self.lX[-1]])
        exo.append(u'\\item ')
        exo.append(r'\begin{itemize}[leftmargin=*]')
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{maximum} de $f$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[1][1], extr[1][0]))
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{minimum} de $f$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[0][1], extr[0][0]))
        exo.append(r'\end{itemize}')
        extr = extrema(self.lX, self.lY1, self.intervalle1)
        extr = extr[self.extremum == "maximum"]
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$, le \\textbf{%s} de $f$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$. ') % (self.intervalle1[0], self.intervalle1[1], self.extremum, extr[1], extr[0]))
        extr = extrema(self.lX, self.lY2, [self.lX[0], self.lX[-1]])
        exo.append(u'\\item ')
        exo.append(r'\begin{itemize}[leftmargin=*]')
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{maximum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[1][1], extr[1][0]))
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{minimum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[0][1], extr[0][0]))
        exo.append(r'\end{itemize}')
        extr = extrema(self.lX, self.lY2, self.intervalle2)
        exo.append(u'\\item ')
        exo.append(r'\begin{itemize}[leftmargin=*]')
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{maximum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.intervalle2[0], self.intervalle2[1], extr[1][1], extr[1][0]))
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{minimum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.intervalle2[0], self.intervalle2[1], extr[0][1], extr[0][0]))
        exo.append(r'\end{itemize}')
        exo.append(r'\end{enumerate}')
        return "\n".join(exo)

class Vf3VariationVersCourbe(ex.TexExercise):
    """Tableaux de variations et courbe"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        lX = [-5, -4, -3, -2, -1, 0, 1, 2, 3, 4, 5]
        self.lX = list(lX)
        self.lY1, self.variations1 = PointsCourbes(sens_var=(-1, 1))
        self.lY2, self.variations2 = PointsCourbes()

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Pour chaque question, répondre avec une phrase en précisant les intervalles.\\vspace{-2ex}'))
        exo.append(r'\begin{multicols}{2}')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Quel est le signe de la fonction $f$ ?'))
        exo.append(_(u'\\item Quels sont les extrema de la fonction $g$ ?'))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{multicols}')
        exo.append(_(u'\\item Tracer une représentation graphique de $f$ et $g$ sur leurs ensembles de définition.'))
        exo.append(r'\end{enumerate}')
        exo.append(r'\begin{center}')
        exo.append(r'\begin{tabular}[t]{ll}')
        for j in range(2):
            variations = [self.variations1, self.variations2][j]
            lY = [self.lY1, self.lY2][j]
            exo.append(r'\begin{tikzpicture}[scale=1]')
            exo.append(r'\tkzTabInit[lgt=1.2,espcl=1.2]{$x$/1,$%s\,(x)$/3}{$%s$}' % (['f', 'g'][j], '$,$'.join([str(v[1]) for v in variations] + [str(variations[-1][2])])))
            ans = r'\tkzTabVar{'  # %-/$-2$,+/$5$,-/$-3$,+/$-1$,-/$-4$}'
            trois_niv = False
            for index in range(len(variations)):
                yindex = variations[index][1] + 5
                if variations[index][0] == 1:
                    if trois_niv:
                        ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                        trois_niv = False
                    else: ans += r'-/$%s$,' % lY[yindex]
                if variations[index][0] == -1:
                    if trois_niv:
                        ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                        trois_niv = False
                    else: ans += r'+/$%s$,' % lY[yindex]
                if variations[index][0] == 0:
                    if index > 0 and index < len(variations) - 1 and variations[index - 1][0] * variations[index + 1][0] > 0:
                        trois_niv = True
                    if trois_niv: ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    elif index > 0:
                        if variations[index - 1][0] == 1: ans += r'+/$%s$,' % lY[yindex]
                        else: ans += r'-/$%s$,' % lY[yindex]
                    elif index < len(variations) - 1:
                        if variations[index + 1][0] == 1: ans += r'-/$%s$,' % lY[yindex]
                        else: ans += r'+/$%s$,' % lY[yindex]
            if variations[-1][0] == 1:  ans += r'+/$%s$}' % lY[-1]
            elif variations[-1][0] == -1:  ans += r'-/$%s$}' % lY[-1]
            elif variations[-1][0] == 0:
                if variations[-2][0] == 1:  ans += r'+/$%s$}' % lY[-1]
                elif variations[-2][0] == -1:  ans += r'-/$%s$}' % lY[-1]
            exo.append(ans)
            # Place les zéros
            for absc in range(-5, 6):
                if lY[absc + 5] == 0:
                    for index in range(len(variations)):
                        if variations[index][1] < absc < variations[index][2]:
                            if variations[index][0] != 0:
                                exo.append(r'\tkzTabVal{%s}{%s}{0.5}{\scriptsize $%s$}{$0$}' % (index + 1, index + 2, absc))
                            break
                        if variations[index][1] == absc or variations[index][2] == absc:
                            break
            exo.append(r'\end{tikzpicture}')
            exo.append(r'&')
        exo.pop(-1)
        exo.append(r'\end{tabular}')
        exo.append(r'\end{center}')

        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        exo.append(r'\item')
        exo.append(r'\begin{enumerate}')
        lneg, lpos = [], []
        for index in range(len(self.lX)):
            absc = self.lX[index]
            if self.lY1[index] < 0:
                if lneg and absc - lneg[-1][1] == 1:
                    lneg[-1][1] = absc
                else:
                    lneg.append([absc, absc])
            elif self.lY1[index] > 0:
                if lpos and absc - lpos[-1][1] == 1:
                    lpos[-1][1] = absc
                else:
                    lpos.append([absc, absc])
            else:  # nul
                if lneg and absc - lneg[-1][1] == 1:
                    lneg[-1][1] = absc
                    lpos.append([absc, absc])
                elif lpos and absc - lpos[-1][1] == 1:
                    lpos[-1][1] = absc
                    lneg.append([absc, absc])
                else:
                    lneg.append([absc, absc])
                    lpos.append([absc, absc])
        sneg, spos = '', ''
        for i in lneg:
            if i[1] == i[0]: lneg.remove(i)
            else: sneg += '$\interval{%s}{%s}$, ' % (i[0], i[1])
        sneg = sneg[:-2]
        for i in lpos:
            if i[1] == i[0]: lpos.remove(i)
            else: spos += '$\interval{%s}{%s}$, ' % (i[0], i[1])
        spos = spos[:-2]

        exo.append(_(u'\\item La fonction $f$ est \\textbf{négative} sur %s et \\textbf{positive} sur %s.') % (sneg, spos))
        extr = extrema(self.lX, self.lY2, [self.lX[0], self.lX[-1]])
        exo.append(r'\item')
        exo.append(r'\begin{itemize}[leftmargin=*]')
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{maximum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[1][1], extr[1][0]))
        exo.append(_(u'\\item Sur $\interval{%s}{%s}$ , le \\textbf{minimum} de $g$ est $y = %s$. Il est \\textbf{atteint en} $x = %s$.') % (self.lX[0], self.lX[-1], extr[0][1], extr[0][0]))
        exo.append(r'\end{itemize}')
        exo.append(r'\end{enumerate}')
        exo.append(r'\item\ ')
        exo.append(r'\begin{center}')
        exo.append(r'\begin{tabular}[t]{cc}')
        exo.append(r'\begin{adjustbox}{valign=t}')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        lAbsc, lOrdo = integration_zeros(self.lX, self.lY1, self.variations1)
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in lAbsc])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in lOrdo])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_f$", (%s,%s), align=%s);' % (self.lX[0], self.lY1[0], ('SE', 'NW', 'SW')[self.variations1[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\end{adjustbox}')
        exo.append(r'&')
        exo.append(r'\begin{adjustbox}{valign=t}')
        exo.append('\\begin{asy}\n import graph;\n import interpolate;\n unitsize(5mm);')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt,ypt;')
        lAbsc, lOrdo = integration_zeros(self.lX, self.lY2, self.variations2)
        exo.append(r'real [] xpt={%s};' % (",".join([str(a).zfill(2) for a in lAbsc])))
        exo.append(r'real [] ypt={%s};' % (",".join([str(a).zfill(2) for a in lOrdo])))
        exo.append(r'xlimits(-6.2, 6.2);')
        exo.append(r'ylimits(-5.2, 5.2);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'draw(graph(xpt,ypt,Hermite(monotonic)),brown+1.5);')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label(L="$\mathcal{C}_g$", (%s,%s), align=%s);' % (self.lX[0], self.lY2[0], ('SE', 'NW', 'SW')[self.variations2[0][0]]))
        exo.append(r'\end{asy}')
        exo.append(r'\end{adjustbox}')
        exo.append(r'\end{tabular}')
        exo.append(r'\end{center}')
        exo.append(r'\end{enumerate}')
        return "\n".join(exo)

def elements_intervalle(variations, sens):
    """Renvoie deux nombres appartenant à un intervalle sur lequel la fonction est monotone, ainsi que l'intervalle monotone auquel ils appartiennent"""
    while True:
        index = randrange(len(variations))
        if variations[index][0] == sens:
            break
    x0 = decimaux(randrange(50 * variations[index][1] // 6 + 10 * variations[index][2] // 6, 20 * variations[index][1] // 3 + 10 * variations[index][2] // 3 + 1) / 10, True)
    x1 = decimaux(randrange(10 * variations[index][1] // 3 + 20 * variations[index][2] // 3, 10 * variations[index][1] // 6 + 50 * variations[index][2] // 6, +1) / 10, True)
    return [x0, x1, [variations[index][1], variations[index][2]]]

def non_monotone(variations, lX, lY, signe):
    """Donne un intervalle sur lequel la fonction n'est pas monotone. On peut malgré tout comparer
    les extrémités de cet intervalle par le signe de leurs images si signe=True"""
    for dummy in range(20):
        tirage = [a for a in range(len(variations))]
        try:
            i0 = tirage.pop(randrange(len(tirage)))
            while lY[i0] == lY[i0 + 1]:
                # intervalle constant
                i0 = tirage.pop(randrange(len(tirage)))
            i1 = tirage.pop(randrange(len(tirage)))
            while lY[i1] == lY[i1 + 1]:
                # intervalle constant
                i1 = tirage.pop(randrange(len(tirage)))
        except ValueError:
            return None
        x0 = randrange(10 * variations[i0][1] + 1, 10 * variations[i0][2]) / 10
        x1 = randrange(10 * variations[i1][1] + 1, 10 * variations[i1][2]) / 10
        for i in range(len(lX)):
            if x0 <= lX[i]:
                if x0 == lX[i]: y0 = lY[i]
                else: y0 = (lY[i - 1] + lY[i]) / 2
                break
        for i in range(len(lX)):
            if x1 <= lX[i]:
                if x1 == lX[i]: y1 = lY[i]
                else: y1 = (lY[i - 1] + lY[i]) / 2
                break
        if signe and y0 * y1 <= 0: return ([x0, y0], [x1, y1])
        if not signe and y0 * y1 > 0:
            I = intersection_intervalles(variations[i0][1:], variations[i1][1:])
            if I and I[0] != I[1]: return ([x0, y0], [x1, y1])
    return None

def intersection_intervalles(I0, I1):
    "renvoie l'intersection entre 2 intervalles, None si vide."
    x0, x1 = min(I0), max(I0)
    y0, y1 = min(I1), max(I1)
    x = max(x0, x1)
    y = min(y0, y1)
    if x <= y:return (x, y)
    else: return None

class Vf4ComparerImages(ex.TexExercise):
    """Comparer des images à partir du sens de variation"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        fin = False
        while not fin:
            lX = [i for i in range(-10 + randrange(6), 10 - randrange(6))]
            self.lX = list(lX)
            lY, variations = PointsCourbes(nb_var=(4, 5), absc=(lX[0], lX[-1]), ordo=(lX[0], lX[-1]), cstes=(1, 1))
            self.lY = list(lY)
            self.variations = list(variations)
            lx = elements_intervalle(self.variations, -1)
            lx.append(-1)
            comparaison = [lx]
            lx = elements_intervalle(self.variations, 1)
            lx.append(1)
            comparaison.append(lx)
            lx = elements_intervalle(self.variations, 0)
            lx.append(0)
            comparaison.append(lx)
            shuffle(comparaison)
            self.comparaison = comparaison
            non_comparable = [non_monotone(self.variations, lX, self.lY, True)]
            non_comparable.append(non_monotone(self.variations, lX, self.lY, False))
            if non_comparable[0] != None and non_comparable[1] != None:
                fin = True
        shuffle(non_comparable)
        self.non_comparable = non_comparable

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item À partir du tableau de variation ci-dessous, recopier et compléter les égalités ou inégalités suivantes en justifiant :\\vspace{-2ex}'))
        exo.append(r'\begin{multicols}{3}')
        exo.append(r'\begin{enumerate}')
        exo.append(r'\item $f\,(%s) \ldots{} f\,(%s)$' % tuple(self.comparaison[0][:2]))
        exo.append(r'\item $f\,(%s) \ldots{} f\,(%s)$' % tuple(self.comparaison[1][:2]))
        exo.append(r'\item $f\,(%s) \ldots{} f\,(%s)$' % tuple(self.comparaison[2][:2]))
        exo.append(r'\end{enumerate}\vspace{-2ex}')
        exo.append(r'\end{multicols}')
        exo.append(_(u'\\item Peut-on comparer l’image des nombres $%s$ et $%s$ ? Justifier.') % (decimaux(self.non_comparable[0][0][0]), decimaux(self.non_comparable[0][1][0])))
        exo.append(_(u'\\item Peut-on comparer l’image des nombres $%s$ et $%s$ ? Justifier.') % (decimaux(self.non_comparable[1][0][0]), decimaux(self.non_comparable[1][1][0])))
        exo.append(r'\end{enumerate}')
        exo.append(r'\begin{center}')
        variations = list(self.variations)
        lY = list(self.lY)
        exo.append(r'\begin{tikzpicture}[scale=1]')
        exo.append(r'\tkzTabInit[lgt=1.2,espcl=2]{$x$/1,$f\,(x)$/3}{$%s$}' % ('$,$'.join([str(v[1]) for v in variations] + [str(variations[-1][2])])))
        ans = r'\tkzTabVar{'  # %-/$-2$,+/$5$,-/$-3$,+/$-1$,-/$-4$}'
        trois_niv = False
        for index in range(len(variations)):
            yindex = variations[index][1] - self.lX[0]
            if variations[index][0] == 1:
                if trois_niv:
                    ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    trois_niv = False
                else: ans += r'-/$%s$,' % lY[yindex]
            if variations[index][0] == -1:
                if trois_niv:
                    ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    trois_niv = False
                else: ans += r'+/$%s$,' % lY[yindex]
            if variations[index][0] == 0:
                if index > 0 and index < len(variations) - 1 and variations[index - 1][0] * variations[index + 1][0] > 0:
                    trois_niv = True
                if trois_niv: ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                elif index > 0:
                    if variations[index - 1][0] == 1: ans += r'+/$%s$,' % lY[yindex]
                    else: ans += r'-/$%s$,' % lY[yindex]
                elif index < len(variations) - 1:
                    if variations[index + 1][0] == 1: ans += r'-/$%s$,' % lY[yindex]
                    else: ans += r'+/$%s$,' % lY[yindex]
        if variations[-1][0] == 1:  ans += r'+/$%s$}' % lY[-1]
        elif variations[-1][0] == -1:  ans += r'-/$%s$}' % lY[-1]
        elif variations[-1][0] == 0:
            if variations[-2][0] == 1:  ans += r'+/$%s$}' % lY[-1]
            elif variations[-2][0] == -1:  ans += r'-/$%s$}' % lY[-1]
        exo.append(ans)
        # Place les zéros
        for absc in range(self.lX[0], self.lX[-1] + 1):
            if lY[absc - self.lX[0]] == 0:
                for index in range(len(variations)):
                    if variations[index][1] < absc < variations[index][2]:
                        if variations[index][0] != 0:
                            exo.append(r'\tkzTabVal{%s}{%s}{0.5}{\scriptsize $%s$}{$0$}' % (index + 1, index + 2, absc))
                        break
                    if variations[index][1] == absc or variations[index][2] == absc:
                        break
        exo.append(r'\end{tikzpicture}')
        exo.append(r'\end{center}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        exo.append(r'\item')
        exo.append(r'\begin{enumerate}')
        for i in range(3):
            exo.append(_(u'\\item $f\\,(%s) %s f\\,(%s)$ car $%s < %s$ et $f$ est %s sur $\interval{%s}{%s}$.') % (self.comparaison[i][0], ['=', '<', '>'][self.comparaison[i][3]], self.comparaison[i][1], self.comparaison[i][0], self.comparaison[i][1], [_(u'constante'), _(u'croissante'), _(u'décroissante')][self.comparaison[i][3]], self.comparaison[i][2][0], self.comparaison[i][2][1]))
        exo.append(r'\end{enumerate}')
        for i in range(2):
            if self.non_comparable[i][0][1] * self.non_comparable[i][1][1] <= 0:
                exo.append(_(u'\\item $f\\,(%s) %s f\\,(%s)$ car d’après le signe de la fonction $f\\,(%s) %s 0$ et $f\\,(%s) %s 0$ (par contre, on ne peut pas utiliser le sens de variation qui change sur l’intervalle $\interval{%s}{%s}$).') % \
                           (decimaux(self.non_comparable[i][0][0], 1), ['<', '>'][self.non_comparable[i][0][1] > 0], decimaux(self.non_comparable[i][1][0], 1), \
                            decimaux(self.non_comparable[i][0][0], 1), ['<', '>'][self.non_comparable[i][0][1] > 0], decimaux(self.non_comparable[i][1][0], 1), \
                            ['<', '>'][self.non_comparable[i][1][1] > 0], decimaux(min(self.non_comparable[i][0][0], self.non_comparable[i][1][0]), 1), decimaux(max(self.non_comparable[i][0][0], self.non_comparable[i][1][0]), 1)))
            else:
                exo.append(_(u'\\item On ne peut pas comparer $f\\,(%s)$ et $f\\,(%s)$ car la fonction $f$ n\'est pas monotone (elle change de sens de variation) sur $\interval{%s}{%s}$.') % \
                           (decimaux(self.non_comparable[i][0][0], 1), decimaux(self.non_comparable[i][1][0], 1), decimaux(self.non_comparable[i][0][0], 1), decimaux(self.non_comparable[i][1][0], 1)))
        exo.append(r'\end{enumerate}')
        return "\n".join(exo)

def extrema_locaux(variations, lX, lY):
    """Donne un intervalle sur lequel la fonction n'est pas monotone et ou les extrema
    sont différents de ceux sur l'ensemble de définition."""
    [gmin, gmax] = extrema(lX, lY, [lX[0], lX[-1]])
    gmin = gmin[1]
    gmax = gmax[1]
    for i in range(20):
        tirage = [a for a in range(len(variations))]
        i0 = tirage.pop(randrange(len(tirage)))
        i1 = tirage.pop(randrange(len(tirage)))
        x0 = randrange(10 * variations[i0][1] + 1, 10 * variations[i0][2]) / 10
        x1 = randrange(10 * variations[i1][1] + 1, 10 * variations[i1][2]) / 10
        for i in range(len(lX)):
            if x0 <= lX[i]:
                if x0 == lX[i]: y0 = lY[i]
                else: y0 = (lY[i - 1] + lY[i]) / 2
                break
        for i in range(len(lX)):
            if x1 <= lX[i]:
                if x1 == lX[i]: y1 = lY[i]
                else: y1 = (lY[i - 1] + lY[i]) / 2
                break
        extremums = [100, -100]
        # extremums = [min(variations[i0][1], variations[i1][1]), max(variations[i0][2], variations[i1][2])]
        if i0 > i1: i0, x0, y0, i1, x1, y1 = i1, x1, y1, i0, x0, y0
        if y0 < 0 and variations[i0][0] == -1:
            extremums[1] = min(lY[variations[i0][1] - lX[0]], 0)
            extremums[0] = min(lY[variations[i0][1] - lX[0]], lY[variations[i0][2] - lX[0]])
        elif y0 > 0 and variations[i0][0] == 1:
            extremums[0] = max(lY[variations[i0][2] - lX[0]], 0)
            extremums[1] = max(lY[variations[i0][1] - lX[0]], lY[variations[i0][2] - lX[0]])
        else:
            extremums[1] = max(lY[variations[i0][1] - lX[0]], lY[variations[i0][2] - lX[0]])
            extremums[0] = min(lY[variations[i0][1] - lX[0]], lY[variations[i0][2] - lX[0]])
        if y1 < 0 and variations[i1][0] == 1: extremums[1] = max(min(0, lY[variations[i1][2] - lX[0]]), extremums[1])
        elif y1 > 0 and variations[i1][0] == -1: extremums[0] = min(max(0, lY[variations[i1][2] - lX[0]]), extremums[0])
        else:
            extremums[1] = max(lY[variations[i1][1] - lX[0]], lY[variations[i1][2] - lX[0]], extremums[1])
            extremums[0] = min(lY[variations[i1][1] - lX[0]], lY[variations[i1][2] - lX[0]], extremums[0])
        for index in range(i0 + 1, i1):
            extremums[0] = min(extremums[0], extrema(lX, lY, variations[index][1:])[0][1])
            extremums[1] = max(extremums[1], extrema(lX, lY, variations[index][1:])[1][1])
        if extremums[0] != gmin and extremums[1] != gmax:
            return [[x0, y0], [x1, y1], extremums]
    return None

class Vf5Extrema_Tableau(ex.TexExercise):
    """Extrema locaux à partir du tableau de variation"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        fin = False
        while not fin:
            lX = [i for i in range(-10 + randrange(6), 10 - randrange(6))]
            self.lX = list(lX)
            lY, variations = PointsCourbes(nb_var=(4, 5), absc=(lX[0], lX[-1]), ordo=(lX[0], lX[-1]), cstes=(1, 1))
            self.lY = list(lY)
            self.variations = list(variations)
            comparaison = [extrema_locaux(variations, lX, lY)]
            comparaison.append(extrema_locaux(variations, lX, lY))
            if comparaison[0] != None and comparaison[1] != None:
                fin = True
        self.comparaison = comparaison
        inegalites = ['\\geqslant{}', '\\leqslant{}']
        shuffle(inegalites)
        inegalites.append(['\\leqslant{}', '\\geqslant{}'][randrange(2)])
        self.inegalites = inegalites

    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item À partir du tableau de variation de la fonction $f$, compléter les égalités ou inégalités suivantes :\\vspace{-2ex}'))
        exo.append(r'\begin{multicols}{2}')
        exo.append(r'\begin{enumerate}')
        exo.append(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s \ldots{}$' % (self.lX[0], self.lX[-1], self.inegalites[0]))
        exo.append(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s \ldots{}$' % (self.lX[0], self.lX[-1], self.inegalites[1]))
        exo.append(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s \ldots{}$' % (decimaux(self.comparaison[0][0][0], 1), decimaux(self.comparaison[0][1][0], 1), self.inegalites[2]))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{multicols}')
        exo.append(u'\\item')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Donner un encadrement de la fonction $f$ sur l’intervalle $\interval{%s}{%s}$.') % (self.lX[0], self.lX[-1]))
        exo.append(_(u'\\item Donner un encadrement de la fonction $f$ sur l’intervalle $\interval{%s}{%s}$.') % (decimaux(self.comparaison[1][0][0], 1), decimaux(self.comparaison[1][1][0], 1)))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{enumerate}')
        exo.append(r'\begin{center}')
        variations = list(self.variations)
        lY = list(self.lY)
        exo.append(r'\begin{tikzpicture}[scale=1]')
        exo.append(r'\tkzTabInit[lgt=1.2,espcl=2]{$x$/1,$f\,(x)$/3}{$%s$}' % ('$,$'.join([str(v[1]) for v in variations] + [str(variations[-1][2])])))
        ans = r'\tkzTabVar{'  # %-/$-2$,+/$5$,-/$-3$,+/$-1$,-/$-4$}'
        trois_niv = False
        for index in range(len(variations)):
            yindex = variations[index][1] - self.lX[0]
            if variations[index][0] == 1:
                if trois_niv:
                    ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    trois_niv = False
                else: ans += r'-/$%s$,' % lY[yindex]
            if variations[index][0] == -1:
                if trois_niv:
                    ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                    trois_niv = False
                else: ans += r'+/$%s$,' % lY[yindex]
            if variations[index][0] == 0:
                if index > 0 and index < len(variations) - 1 and variations[index - 1][0] * variations[index + 1][0] > 0:
                    trois_niv = True
                if trois_niv: ans += r'+/ \raisebox{-2.7cm}{}\raisebox{-1.5cm}{$%s$},' % lY[yindex]
                elif index > 0:
                    if variations[index - 1][0] == 1: ans += r'+/$%s$,' % lY[yindex]
                    else: ans += r'-/$%s$,' % lY[yindex]
                elif index < len(variations) - 1:
                    if variations[index + 1][0] == 1: ans += r'-/$%s$,' % lY[yindex]
                    else: ans += r'+/$%s$,' % lY[yindex]
        if variations[-1][0] == 1:  ans += r'+/$%s$}' % lY[-1]
        elif variations[-1][0] == -1:  ans += r'-/$%s$}' % lY[-1]
        elif variations[-1][0] == 0:
            if variations[-2][0] == 1:  ans += r'+/$%s$}' % lY[-1]
            elif variations[-2][0] == -1:  ans += r'-/$%s$}' % lY[-1]
        exo.append(ans)
        # Place les zéros
        for absc in range(self.lX[0], self.lX[-1] + 1):
            if lY[absc - self.lX[0]] == 0:
                for index in range(len(variations)):
                    if variations[index][1] < absc < variations[index][2]:
                        if variations[index][0] != 0:
                            exo.append(r'\tkzTabVal{%s}{%s}{0.5}{\scriptsize $%s$}{$0$}' % (index + 1, index + 2, absc))
                        break
                    if variations[index][1] == absc or variations[index][2] == absc:
                        break
        exo.append(r'\end{tikzpicture}')
        exo.append(r'\end{center}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        exo.append(r'\item')
        exo.append(r'\begin{multicols}{2}')
        exo.append(r'\begin{enumerate}')
        extr = extrema(self.lX, self.lY, [self.lX[0], self.lX[-1]])
        exo.append(_(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s %s$') % (self.lX[0], self.lX[-1], self.inegalites[0], extr['\\leqslant{}' == self.inegalites[0]][1]))
        exo.append(_(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s %s$') % (self.lX[0], self.lX[-1], self.inegalites[1], extr['\\leqslant{}' == self.inegalites[1]][1]))
        exo.append(_(r'\item Pour $x \in \interval{%s}{%s},\quad f\,(x) %s %s$') % (decimaux(self.comparaison[0][0][0], 1), decimaux(self.comparaison[0][1][0], 1), self.inegalites[2], self.comparaison[0][2]['\\leqslant{}' == self.inegalites[2]]))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{multicols}')
        exo.append(r'\item')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Sur $\interval{%s}{%s},\\quad %s \\leqslant{} f\\,(x) \\leqslant{} %s$.') % (self.lX[0], self.lX[-1], extr[0][1], extr[1][1]))
        exo.append(_(u'\\item Sur $\interval{%s}{%s},\\quad %s \\leqslant{} f\\,(x) \\leqslant{} %s$.') % (decimaux(self.comparaison[1][0][0], 1), decimaux(self.comparaison[1][1][0], 1), decimaux(self.comparaison[1][2][0], 1), decimaux(self.comparaison[1][2][1], 1)))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{enumerate}')

        return "\n".join(exo)
